'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var t = require('assert');
var fs = require('fs');
var path = require('path');
var http = require('http');
var stream = require('stream');

var request = require('../../').client;
var file = {
  text: path.resolve(__dirname, './body.js'),
  binary: path.resolve(__dirname, '../fixtures/cat.png')
};

describe('body', function () {
  var server;

  before(function (done) {
    server = http.createServer();
    server.on('request', function (req, res) {
      if (req.url === '/stream') {
        t.equal(req.headers['transfer-encoding'], 'chunked', 'transfer encoding should be chunked');
        req.pipe(res);
      } else {
        t.equal(req.headers['content-length'], '3', 'content-length should be set');
        req.pipe(res);
      }
    });
    server.listen(5000, done);
  });

  it('string', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var _ref2, body;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return request({
              url: 'http://localhost:5000',
              body: 'hey'
            });

          case 2:
            _ref2 = _context.sent;
            body = _ref2.body;

            t.equal(body, 'hey', 'string request body');

          case 5:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  })));

  it('buffer', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
    var _ref4, body;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return request({
              url: 'http://localhost:5000',
              body: Buffer.from('hey')
            });

          case 2:
            _ref4 = _context2.sent;
            body = _ref4.body;

            t.equal(body, 'hey', 'Buffer request body');

          case 5:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));

  it('stream text', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    var _ref6, body;

    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return request({
              url: 'http://localhost:5000/stream',
              body: fs.createReadStream(file.text, { highWaterMark: 1024 }),
              encoding: null
            });

          case 2:
            _ref6 = _context3.sent;
            body = _ref6.body;

            t.equal(fs.statSync(file.text).size, body.length, 'input file size should equal response body length');

          case 5:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  })));

  it('stream binary', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
    var _ref8, body;

    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return request({
              url: 'http://localhost:5000/stream',
              body: fs.createReadStream(file.binary, { highWaterMark: 1024 }),
              encoding: null
            });

          case 2:
            _ref8 = _context4.sent;
            body = _ref8.body;

            t.equal(fs.statSync(file.binary).size, body.length, 'input file size should equal response body length');

          case 5:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  })));

  after(function (done) {
    server.close(done);
  });
});