'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var t = require('assert');
var fs = require('fs');
var path = require('path');
var http = require('http');

var request = require('../').stream;
var file = {
  binary: path.resolve(__dirname, './fixtures/cat.png')
};

describe('stream', function () {
  var server;

  before(function (done) {
    server = http.createServer();
    server.on('request', function (req, res) {
      if (req.url === '/redirect') {
        res.writeHead(301, { location: 'http://localhost:5000' });
        res.end();
      } else {
        req.pipe(res);
      }
    });
    server.listen(5000, done);
  });

  it('file -> req -> file', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var _ref2, res;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return request({
              url: 'http://localhost:5000',
              body: fs.createReadStream(file.binary, { highWaterMark: 1024 })
            });

          case 2:
            _ref2 = _context.sent;
            res = _ref2.res;
            _context.next = 6;
            return new Promise(function (resolve) {
              var body = [];
              res.on('data', function (chunk) {
                body.push(chunk);
              });
              res.on('end', function () {
                t.equal(fs.statSync(file.binary).size, Buffer.concat(body).length, 'input file size should equal response body length');
                resolve();
              });
            });

          case 6:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  })));

  it('file -> req -> req -> file', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
    var _ref4, res1, _ref5, res2;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return request({
              url: 'http://localhost:5000',
              body: fs.createReadStream(file.binary, { highWaterMark: 1024 })
            });

          case 2:
            _ref4 = _context2.sent;
            res1 = _ref4.res;
            _context2.next = 6;
            return request({
              url: 'http://localhost:5000',
              body: res1
            });

          case 6:
            _ref5 = _context2.sent;
            res2 = _ref5.res;
            _context2.next = 10;
            return new Promise(function (resolve) {
              var body = [];
              res2.on('data', function (chunk) {
                body.push(chunk);
              });
              res2.on('end', function () {
                t.equal(fs.statSync(file.binary).size, Buffer.concat(body).length, 'input file size should equal response body length');
                resolve();
              });
            });

          case 10:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));

  it.skip('file -> req -> redirect -> req -> file', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    var _ref7, res;

    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return request({
              url: 'http://localhost:5000/redirect',
              body: fs.createReadStream(file.binary, { highWaterMark: 1024 })
            });

          case 2:
            _ref7 = _context3.sent;
            res = _ref7.res;
            _context3.next = 6;
            return new Promise(function (resolve) {
              var body = [];
              res.on('data', function (chunk) {
                body.push(chunk);
              });
              res.on('end', function () {
                t.equal(fs.statSync(file.binary).size, Buffer.concat(body).length, 'input file size should equal response body length');
                resolve();
              });
            });

          case 6:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  })));

  after(function (done) {
    return server.close(done);
  });
});